@@ -194,7 +194,7 @@ static int my_strcmp(char *windowname, char *name)
      }
 }
 
-static char check_window_name(Window w, char *name)
+static char check_window_name(Window w, char *name, int (*cb)(Window))
 {
      char *windowname;
      XClassHint windowclass;
@@ -207,7 +207,9 @@ static char check_window_name(Window w, char *name)
 	    w, windowname, name);
 	  match = !my_strcmp(windowname, name);
 	  XFree(windowname);
-	  if (match) return 1;
+	  if (match) {
+	       return cb(w);
+	  }
      }
      
      D("XGetClassHint\n");
@@ -218,12 +220,14 @@ static char check_window_name(Window w, char *name)
 	  match = !my_strcmp(windowclass.res_name, name);
 	  XFree(windowclass.res_class);
 	  XFree(windowclass.res_name);
-	  if (match) return 1;
+	  if (match) {
+	       return cb(w);
+	  }
      }
      return 0;
 }
 
-static Window winrecur(Window from, int depth, char *name)
+static Window winrecur(Window from, int depth, char *name, int (*cb)(Window))
 {
      Window root, parent;
      Window *children;
@@ -233,10 +237,10 @@ static Window winrecur(Window from, int depth, char *name)
      
      if (!(flags & H_HIDDEN) && !is_viewable(display, from))
 	  return (Window)0;
-     
-     if (check_window_name(from, name))
+   
+     if (check_window_name(from, name, cb))
 	  return from;
-     
+
      if (!XQueryTree(display, from, &root, &parent, &children, &num_children))
      {
 	  D("Querytree failed!!!\n");
@@ -248,7 +252,7 @@ static Window winrecur(Window from, int depth, char *name)
 	  D("Num children = %d\n", num_children);
 	  for (e = 0; e < num_children; e++)
 	  {
-	       Window win = winrecur(children[e], depth-1, name);
+	       Window win = winrecur(children[e], depth-1, name, cb);
 	       if (win)
 	       {
 		    XFree(children);
@@ -261,6 +265,36 @@ static Window winrecur(Window from, int depth, char *name)
      return (Window)0;
 }
 
+#define MAX_IGNORED 3000
+static int windows_found = 0;
+static Window ignored[MAX_IGNORED];
+
+int initial_win_cb(Window w)
+{
+     if (windows_found < MAX_IGNORED) {
+	  ignored[windows_found++] = w;
+	  D("Ignoring window %x\n", w);
+     }
+     return 0;
+}
+
+/* Returns 1 if and only if w is a new window. */
+int win_cb(Window w)
+{
+  int i;
+  for (i=0; i < windows_found; i++) {
+       if (ignored[i] == w)
+	    return 0;
+  }
+  return 1;
+}
+
+void save_initial_windows()
+{
+     windows_found = 0;
+     winrecur(wattr.root, 3, winname, initial_win_cb);
+}
+
 static int setup_display()
 {
      char *displayname = getenv("DISPLAY");
@@ -302,15 +336,14 @@ static void run_app(char **argv)
      exit(EX_UNAVAILABLE);
 }
 
-static void find_victim(Window checkme)
+static void find_victim(Window window)
 {
      if (!victim)
      {
 	  D("Looking for victim... (%s)\n", winname);
 	  
-	  victim = winrecur(checkme ? checkme : wattr.root,
-			    checkme ? 2 : 3,
-			    winname);
+	  victim = winrecur(window, 2, winname, win_cb);
+
 	  if (victim)
 	  {
 	       D("Found it!! Victim=%x\n", victim);
@@ -336,7 +369,6 @@ static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
      return True;
 }
 
-
 static void check_x_events(void)
 {
      XEvent ev;
@@ -375,7 +407,6 @@ static void check_x_events(void)
 	       {
 	       case UnmapNotify:
 		    D("UNMAPNOTIFY\n");
-		    XMapWindow(display, victim);
 		    break;
 	    
 	       case MapNotify:
@@ -388,8 +419,8 @@ static void check_x_events(void)
 		    if (ev.xreparent.parent == WINDOW)
 		    {
 			 D("REPARENT NOTIFY to the right window\n");
-			 XMapWindow(display, victim);
 			 adjust_window_size();
+			 XMapWindow(display, victim);
 		    } else {
 			 D("REPARENT NOTIFY to some other window!\n");
 			 place_window();
@@ -402,13 +433,14 @@ static void check_x_events(void)
 	       switch (ev.type)
 	       {
 	       case ConfigureRequest:
+		    /* The application has modified its window. */
 		    D("ConfigureRequest\n");
-		    if (set_aspect(ev.xconfigure.width, ev.xconfigure.height))
-			 adjust_window_size();
+		    set_aspect(ev.xconfigure.width, ev.xconfigure.height);
+		    adjust_window_size();
 		    break;
 
 	       case DestroyNotify:
-		    /* We probably need to find a new victim */
+		    D("DestroyNotify\n");
 		    break;
 	       }
 	  }
@@ -448,7 +480,7 @@ static void check_pipe_fd_events(void)
      {
 	  XSelectInput(display, WINDOW, 
 		       SubstructureNotifyMask |
-		       SubstructureRedirectMask );
+		       SubstructureRedirectMask);
 	  XSync(display, False);
      }
   
@@ -593,8 +625,12 @@ int main(int argc, char **argv)
 	  else
 	  {
                /* Has to be done before forking */
-	       XSelectInput(display, WINDOW, SubstructureNotifyMask | SubstructureRedirectMask);
-	       XSelectInput(display, wattr.root, SubstructureNotifyMask);
+	       save_initial_windows();
+	       XSelectInput(display, WINDOW,
+			    SubstructureNotifyMask |
+			    SubstructureRedirectMask);
+	       XSelectInput(display, wattr.root,
+			    SubstructureNotifyMask);
 	       XSync(display, False);
 	  }
 
