@@ -1109,10 +1109,12 @@ NPError NPP_NewStream(NPP instance,
      {
 	  *stype = NP_NORMAL;
 	  new_child(instance, stream->url);
-	  return NPERR_GENERIC_ERROR;
+     }
+     else
+     {
+	  *stype = NP_ASFILEONLY;
      }
 
-     *stype = NP_ASFILEONLY;
      return NPERR_NO_ERROR;
 }
 
@@ -1205,7 +1207,7 @@ int32 NPP_Write(NPP instance,
 		int32 len,
 		void *buf)
 {
-     D("Write\n");
+     D("Write %d\n", len);
      return len;
 }
 
@@ -1213,6 +1215,11 @@ int32 NPP_WriteReady(NPP instance, NPStream *stream)
 {
      D("WriteReady\n");
 
-     if (!instance) return 0;
+     if (!instance) {
+	  D("No instance\n");
+	  return 0;
+     }
+
+     D("Return MAXINT\n");
      return MAXINT;
 }
