@@ -16,21 +16,24 @@
 #include <X11/Xatom.h>
 #include <sys/socket.h>
 
+/*****************************************************************************
+ * Defines
+ *****************************************************************************/
+
 #ifndef MAXINT
 #define MAXINT 0x7fffffff
 #endif
 
 #define MAX(X,Y) ((X)>(Y)?(X):(Y))
 
-/*****************************************************************************
- * Timeout for select()
- *****************************************************************************/
+/* Timeout for select() */
 #define SELECT_TIMEOUT_SEC  0
 #define SELECT_TIMEOUT_USEC 100*1000
 
-/*****************************************************************************
- * Flags
- *****************************************************************************/
+/* Time to wait for a process to exit */
+#define KILL_TIMEOUT_USEC 100000
+
+/* Flags */
 #define H_LOOP 		0x1
 #define H_DAEMON 	0x2
 #define H_STREAM 	0x4
